#!/usr/bin/env python
###############################
# ap.py Platform Module
####


############
# TODO: Should this whole thing be a class?
# __init__.py should do RhevCoordinator().run()
#####

###########
# Imports
####
import os
import time
import json

from redhat_access_insights import parse_config_file
from redhat_access_insights.collection_rules import InsightsConfig
from redhat_access_insights.connection import InsightsConnection


############################
# Run the platform module
# Main execution function
# Called from the main coordinator
#####
def run(options, constants, utilities, logger, insights_uploader_conf, config):
    logger.vlog("Running AP Insights Coordinator")

    starttime = time.time()

    # create config file for client
    rhai_config = parse_config_file('/etc/redhat-access-insights/redhat-access-insights.conf')
    rules_config = InsightsConfig(rhai_config, InsightsConnection(rhai_config))
    collection_rules = rules_config.get_collection_rules(raw=True)
    sig = rules_config.fetch_gpg()
    config = {"sig": sig, "uploader.json": collection_rules}

    config = json.dumps(config)

    # save new config
    try:
        os.makedirs("/var/tmp/redhat-insights")
    except OSError:
        pass

    insights_config_file_name = "/var/tmp/redhat-insights/client.conf"
    insights_config_file = open(insights_config_file_name, "wb")
    insights_config_file.write(config)
    insights_config_file.close()

    # run insights on all the nodes (let ansible do the heavy lifting)
    utilities.run_command('ansible-playbook /var/lib/redhat-insights/insights.yml')

    # Tar everything up and clean
    utilities.compress_archives()

    # Upload to red hat
    options.upload = True
    if options.upload:
        thetime=time.time()-starttime
        upload_success = utilities.upload_archives(utilities.get_tmpgz(), thetime)
        if upload_success and options.clean:
            utilities.clean_tmps()
    else:
        logger.vlog("Archive saved to %s." % (utilities.get_tmpgz()))


# def other_crap():
#     # Setup SSH
#     ssh_client = RhevCoordinatorSSH(constants,logger,config)
#
#     # Setup Manager Options
#     manager_options = constants.api_options
#     # TODO: Should probably move this if logic into the rhevm coordinator
#     if options.api_options:
#         if options.api_options == 'none':
#             logger.vlog("We are not collecting any metadata from the RHEV Manger API, 'none' was passed in.")
#             manager_options = []
#         elif options.api_options == 'all':
#             logger.vlog("We are collecting all metadata from the RHEV Manager API, 'all' was passed in.")
#             manager_options = constants.api_options
#         else:
#             manager_options = options.api_options.split(',')
#             logger.vlog("We are collecting specific metadata from the RHEV Manager API...")
#             logger.vlog(manager_options)
#     rhevm = RhevCoordinatorRhevm(constants, logger, ssh_client, utilities, config)
#     rhevm.collect_manager_information(manager_options)
#
#     # Get all hypervisors/ips
#     logger.vlog("Preparing to gather hypervisors from the manager...")
#     hypervisor_list = rhevm.get_hypervisor_list()
#     rhevm.print_hypervisor_list(hypervisor_list)
#
#     # If we are only testing if the hypervisors are alive and
#     # have a valid SSH connection
#     if options.test_hypervisors:
#         sys.exit("Only running tests for valid hypervisors or valid SSH connections. Exiting.")
#
#     # Run Insights on the manager
#     # We run Insights on the manager first so that the Collection Rules are automatically updated.
#     # This ensures that each subsequent configuration file that is passed to the individual hypervisors contains updated rules
#     rhevm.run_insights_on_manager()
#
#     # FOR ALL HYPERVISORS
#     # 1) ssh into each hypervisor/ip and install latest client in /tmp, this will be removed once we have the insights RPM on the hypervisor image
#     # 2) run latest client on each hypervisor in /tmp, will be modified to run native installed insights rpm once we have it on the images
#     # 3) get stdout from client
#     # 4) clean /tmp
#     stdout = ""
#     starttime = time.time()
#     for hypervisor in hypervisor_list:
#         if hypervisor['is_alive'] and hypervisor['can_ssh']:
#             rhevm.get_rhevh().install_hypervisor(hypervisor['ip'], options)
#             stdout = rhevm.get_rhevh().run_hypervisor(hypervisor['ip'], hypervisor['id'], hypervisor['name'])
#             utilities.write_archive(stdout)
#             if options.clean:
#                 rhevm.get_rhevh().clean_hypervisor(hypervisor['ip'])
#
#     # Tar everything up and clean
#     utilities.compress_archives()
#
#     # Upload to red hat
#     if options.upload:
#         thetime=time.time()-starttime
#         upload_success = utilities.upload_archives(utilities.get_tmpgz(),thetime)
#         if upload_success and options.clean:
#             utilities.clean_tmps()
#     else:
#         logger.vlog("Archive saved to %s." % (utilities.get_tmpgz()))

############
# Main
#####
if __name__ == "__main__":
    import doctest
    doctest.testmod()