#!/usr/bin/env python

##########
# Imports
####
import optparse


class APCoordinatorParser:

    # Containers
    parser = None
    constants = None
    config = None
    options = None

    # Constructor
    def __init__(self, ap_constants, ap_config):
        self.constants = ap_constants
        self.config = ap_config
        self.setup_options()
        self.setup_parser()

    # Setup the Options
    def setup_options(self):
        self.options = [

            # Make this thing interactive
            # This is for debugging, goes through the logger and asks you if you want to continue to the next step
            optparse.make_option('-i', '--interactive', dest='interactive',
                                 help="This must be flagged with --verbose. This prompts for the execution of each step.",
                                 action='store_true',
                                 default=self.config.get_config_var('interactive')),

            # Make things Verbose
            optparse.make_option('-v', '--verbose', dest='verbose',
                                 help='In addition to logging to the Coordinator Logs, print output to the console.',
                                 action='store_true',
                                 default=self.config.get_config_var('verbose')),

            # Archive Only
            optparse.make_option('--archive-only', dest='archive_only',
                                 help='Only create the Coordinator Archive in /tmp. This will not upload the final Archive to Red Hat.',
                                 action='store_true',
                                 default=self.config.get_config_var('archive_only')),
# start rhev
            # Test API
            optparse.make_option('--test-api', dest='test_api',
                                 help='Only test the AP Manager API Connection. Coordinator will exit after test.',
                                 action='store_true',
                                 default=self.constants.test_api),

            # Register API
            optparse.make_option('--register-api', dest='register_api',
                                 help='This will run the Coordinator API Registration Utility. Will register and configure a new AP Insights Coordinator specific user. Overwrites any exisiting API User configured in /etc/redhat-access-insights/insights-coordinator-ap.conf configuration file.',
                                 action='store_true',
                                 default=self.constants.register_api),

            # API User
            optparse.make_option('--api-user', dest='api_user',
                                 help="Will allow using a specific RHEV API User during this execution of the coordinator. Will temporarily override any existing API User configured.",
                                 default=self.config.get_config_var('api_username')),

            # API Password
            optparse.make_option('--api-password', dest='api_password',
                                 help="Will prompt for a RHEV API password during this execution of the coordinator. Will temporarily override any existing API Password configured.",
                                 default=self.config.get_config_var('api_password')),

            # API URL
            optparse.make_option('--api-url', dest='api_url',
                                 help='Will allow using a specific RHEV API URL during this execution of the coordinator. Will temporarily override any existing API URL configured.',
                                 default=self.config.get_config_var('api_url')),

            # API Options
            optparse.make_option('--api-options', dest='api_options',
                                 help='Will allow using specific RHEV API Options for retrieval from the API. Will temporarily override any existing API Options configured.',
                                 default=self.config.get_config_var('api_options')),

            # Test Hypervisors
            optparse.make_option('--test-hypervisors', dest='test_hypervisors',
                                 help='Will only test the Hypervisors connected to the RHEV Manager. The coordinator will exit after testing all Hypervisors.',
                                 action='store_true',
                                 default=self.constants.test_hypervisors),
# end rhev
            # Disable Cron
            optparse.make_option('--disable-cron', dest='disable_cron',
                                 help='Will unlink and disable the Coordinator Cron Job.',
                                 action='store_true',
                                 default=self.constants.disable_cron),

            # Enable Cron
            optparse.make_option('--enable-cron', dest='enable_cron',
                                 help='Will link and enable the Coordinator Cron Job.',
                                 action='store_true',
                                 default=self.config.get_config_var('enable_cron')),

            # SSH User
            optparse.make_option('--ssh-user', dest='ssh_user',
                                 help='Will allow a specific runtime SSH User for this execution of the Coordinator to use for each Hypervisor. This will temporarily override any existing configured SSH user in the Coordinator configuration files.',
                                 default=self.config.get_config_var('ssh_user')),

            # SSH Port
            optparse.make_option('--ssh-port', dest='ssh_port',
                                 help='Will allow a specific runtime SSH Port for this execution of the Coordinator to use for each Hypervisor. This will temporarily override any existing configured SSH port in the Coordinator configuration files.',
                                 default=self.config.get_config_var('ssh_port')),

            # SSH Pass
            optparse.make_option('--ssh-pass', dest='ssh_pass',
                                 help='Will prompt for a SSH password to use for this runtime of the Coordinator for each Hypervisor. This will temporarily override any currently configured SSH password in the Coordinator configuration files.',
                                 default=self.config.get_config_var('ssh_pass')),

            # SSH Key
            optparse.make_option('--ssh-key', dest='ssh_key',
                                 help='Will allow a specific absolute path to a SSH Key for this runtime of the Coordinator for each Hypervisor. This will temporarily override any currently configured SSH key in the Coordinator configuraiton files.',
                                 default=self.config.get_config_var('ssh_key')),

            # Configure
            optparse.make_option('--configure', dest='configure',
                                 help='Will run the Coordinator configuration utility. This will re-write the Coordinator configuration file to /etc/redhat-access-insights/insights-coordinator-rhev.conf. After running the utility the Coordinator will exit.',
                                 action='store_true',
                                 default=self.constants.configure)
        ]

    # Setup the Parser
    def setup_parser(self):
        self.parser = optparse.OptionParser(usage=self.get_usage(), option_list=self.get_options())

    # Get the options
    def get_options(self):
        return self.options

    # Get the parser usage
    def get_usage(self):
        return self.constants.coordinator_parser_usage

    # Parse options
    def parse_options(self):
        # Parse all of the options
        options = self.parser.parse_args()
        options = options[0]

        # Override any existing configuration values with temporary options
        for option_key in [option for option in dir(options) if not option.startswith('_') and not option.endswith('_')]:
            self.config.set_temp_config_var(option_key, getattr(options, option_key))

        # Return only the parsed options, don't need to return additional arguments
        return options
